Package: gdsfmt
Type: Package
Title: R Interface to CoreArray Genomic Data Structure (GDS) Files
Version: 1.40.1
Date: 2024-08-15
Depends: R (>= 2.15.0), methods
Suggests: parallel, digest, Matrix, crayon, RUnit, knitr, markdown,
        rmarkdown, BiocGenerics
Author: Xiuwen Zheng [aut, cre] (<https://orcid.org/0000-0002-1390-0708>),
        Stephanie Gogarten [ctb],
        Jean-loup Gailly and Mark Adler [ctb] (for the included zlib sources),
        Yann Collet [ctb] (for the included LZ4 sources),
        xz contributors [ctb] (for the included liblzma sources)
Maintainer: Xiuwen Zheng <zhengx@u.washington.edu>
Description: Provides a high-level R interface to CoreArray Genomic Data
        Structure (GDS) data files. GDS is portable across platforms with
        hierarchical structure to store multiple scalable array-oriented data
        sets with metadata information. It is suited for large-scale datasets,
        especially for data which are much larger than the available
        random-access memory. The gdsfmt package offers the efficient operations
        specifically designed for integers of less than 8 bits, since a diploid
        genotype, like single-nucleotide polymorphism (SNP), usually occupies
        fewer bits than a byte. Data compression and decompression are available
        with relatively efficient random access. It is also allowed to read a
        GDS file in parallel with multiple R processes supported by the package
        parallel.
License: LGPL-3
Copyright: This package includes the sources of CoreArray C++ library
        written by Xiuwen Zheng (LGPL-3), zlib written by Jean-loup
        Gailly and Mark Adler (zlib license), LZ4 written by Yann
        Collet (simplified BSD), and liblzma written by Lasse Collin
        and other xz contributors (public domain).
VignetteBuilder: knitr
ByteCompile: TRUE
BugReports: https://github.com/zhengxwen/gdsfmt/issues
URL: https://github.com/zhengxwen/gdsfmt
biocViews: Infrastructure, DataImport
git_url: https://git.bioconductor.org/packages/gdsfmt
git_branch: RELEASE_3_19
git_last_commit: b1a5184
git_last_commit_date: 2024-08-15
Repository: Bioconductor 3.19
Date/Publication: 2024-08-18
NeedsCompilation: yes
Packaged: 2024-08-18 21:44:10 UTC; biocbuild
Built: R 4.4.1; x86_64-apple-darwin13.4.0; 2024-08-23 05:28:46 UTC; unix
